/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  Small opening book. Based on tscp source.
//-------------------------------------------------------------------------------------------------

char    *book_lines[1000];
int     book_count;
size_t  book_max_line;
char    current_line[1024];

int     get_current_line(void);

//-------------------------------------------------------------------------------------------------
//  Return the next book move for current position or MOVE_NONE if there's no match.
//-------------------------------------------------------------------------------------------------
MOVE book_next_move(void)
{
	char    moves[100][10];
	int	    options;
	int     i;
	int     choose;
    MOVE    move;

    if (!get_current_line())
        return 0;

	for(i = options = 0; i < book_count && options < 100; i++)
	{
		char *aux = book_lines[i];
		if (!strncmp(current_line, aux, strlen(current_line)))
        {
			size_t start = strlen(current_line);
			while(aux[start] && aux[start] == ' ')
				start++;
			strncpy(moves[options++], &aux[start], 4);
			moves[options - 1][4] = 0;
		}
	}
	if (options == 0)
		return MOVE_NONE;

	choose = rand() % options;
	util_parse_move(moves[choose], &move);
    if (move == MOVE_NONE || !is_valid(move))
        move = MOVE_NONE;

	return move;
}

//-------------------------------------------------------------------------------------------------
//  Get the current game list of moves so it can search the book.
//-------------------------------------------------------------------------------------------------
int get_current_line(void)
{
	int i;
	char move_string[10];

	current_line[0] = 0;
    for (i = 0; i < histply; i++)
    {
        if (i > 0)
            strcat(current_line, " ");
        util_get_move_string(history[i].move, move_string);
        if (strlen(current_line) + strlen(move_string) > sizeof(current_line))
            return 0;
        strcat(current_line, move_string);
        if (strlen(current_line) > book_max_line)
            return 0;
    }

	return 1;
}

//-------------------------------------------------------------------------------------------------
//  Add a move line to the book.
//-------------------------------------------------------------------------------------------------
void book_add_line(char *line)
{
	book_lines[book_count++] = line;
    if (strlen(line) > book_max_line)
        book_max_line = strlen(line);
}

//-------------------------------------------------------------------------------------------------
//  Init the book lines.
//-------------------------------------------------------------------------------------------------
void book_init(void)
{
    srand((unsigned int)time(0));
    book_count = 0;
    book_max_line = 0;

    book_add_line("d2d4 g8f6 c2c4 g7g6 b1c3 d7d5 g1f3 f8g7 c4d5 f6d5 e2e4 d5c3 b2c3 c7c5 c1e3 d8a5 d1d2 b8c6 a1c1 c5d4");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 b7b6 g2g3 c8a6 b2b3 f8b4 c1d2 b4e7 f1g2 c7c6 e1g1 d7d5 d1c2 b8d7 f1d1 e8g8");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 g8f6 e1g1 f6e4 d2d4 e4d6 b5c6 d7c6 d4e5 d6f5 d1d8 e8d8 b1c3 c8d7 f1d1 d8c8");
    book_add_line("g1f3 c7c5 c2c4 g8f6 b1c3 e7e6 g2g3 b7b6 f1g2 c8b7 e1g1 f8e7 d2d4 c5d4 d1d4 b8c6 d4f4 e8g8 f1d1 d8b8");
    book_add_line("e2e4 e7e5 b1c3 g8f6 g2g3 d7d5 e4d5 f6d5 f1g2 d5c3 b2c3 f8d6 g1f3 e8g8 e1g1 b8d7 d2d4 h7h6 f1e1 e5d4");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 b7b6 g2g3 c8a6 b2b3 f8b4 c1d2 b4e7 f1g2 c7c6 e1g1 d7d5 d1c2 b8d7 f1d1 a8c8");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 g8f6 e1g1 f6e4 d2d4 e4d6 b5c6 d7c6 d4e5 d6f5 d1d8 e8d8 b1c3 c8d7 f1d1 d8c8");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 c7c5 g1e2 c5d4 e3d4 e8g8 a2a3 b4e7 e2f4 d7d5 c4d5 f6d5 c3d5 e6d5");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 e7e6 d4b5 f8b4 a2a3 b4c3 b5c3 d7d5 e4d5 e6d5 f1d3 e8g8");
    book_add_line("d2d4 e7e6 c2c4 c7c5 d4d5 e6d5 c4d5 d7d6 b1c3 g7g6 e2e4 f8g7 g1f3 a7a6 h2h3 b7b5 f1d3 g8e7 e1g1 e8g8");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 e7e6 d4b5 f8b4 a2a3 b4c3 b5c3 d7d5 e4d5 e6d5 f1d3 e8g8");
    book_add_line("g1f3 g8f6 g2g3 d7d5 f1g2 e7e6 e1g1 f8e7 d2d3 e8g8 b1d2 a7a5 a2a4 b7b6 b2b3 c8b7 c1b2 b8d7 e2e3 f8e8");
    book_add_line("g1f3 g8f6 g2g3 b7b6 f1g2 c8b7 e1g1 c7c5 c2c4 e7e6 b1c3 f8e7 f1e1 a7a6 e2e4 d7d6 d2d4 c5d4 f3d4 d8c7");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 d1c2 d7d6 e2e3 e8g8 f1d3 c7c5 d4c5 d6d5 g1f3 b8c6 e1g1 b4c5 a2a3 d8e7");
    book_add_line("g1f3 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 e8g8 f1e2 b7b6 e1g1 d7d5 a2a3 b4c3 b2c3 c8a6 c4d5 a6e2 d1e2 e6d5");
    book_add_line("g1f3 g8f6 c2c4 c7c5 d2d4 c5d4 f3d4 b8c6 g2g3 d8b6 d4c2 e7e6 f1g2 d7d5 c4d5 e6d5 e1g1 c8e6 b1c3 a8d8");
    book_add_line("d2d4 g8f6 c2c4 g7g6 b1c3 f8g7 e2e4 d7d6 g1f3 e8g8 f1e2 c7c5 e1g1 c5d4 f3d4 b8c6 c1e3 c8d7 d1d2 c6d4");
    book_add_line("d2d4 e7e6 c2c4 f7f5 g2g3 g8f6 f1g2 c7c6 b1d2 d7d5 g1h3 f8d6 e1g1 e8g8 d1c2 b7b6 d2f3 f6e4 c1f4 c8b7");
    book_add_line("d2d4 e7e6 c2c4 f7f5 b1c3 g8f6 d1c2 f8b4 e2e3 e8g8 f1d3 d7d6 g1e2 b8c6 e1g1 b4c3 c2c3 a7a5 b2b3 e6e5");
    book_add_line("d2d4 d7d6 e2e4 g8f6 b1c3 g7g6 c1e3 c7c6 f2f3 b7b5 d1d2 b8d7 g2g4 d7b6 h2h4 h7h5 g4g5 f6d7 b2b3 f8g7");
    book_add_line("e2e4 c7c6 d2d4 d7d5 b1d2 d5e4 d2e4 c8f5 e4g3 f5g6 h2h4 h7h6 g1f3 b8d7 h4h5 g6h7 f1d3 h7d3 d1d3 e7e6");
    book_add_line("c2c4 g8f6 b1c3 c7c5 g2g3 e7e6 g1f3 b7b6 f1g2 c8b7 e1g1 f8e7 f1e1 d7d5 c4d5 e6d5 d2d4 e8g8 c1f4 b8d7");
    book_add_line("g1f3 g8f6 d2d4 e7e6 e2e3 b7b6 f1d3 c8b7 c2c4 f8b4 b1d2 e8g8 e1g1 d7d5 a2a3 b4e7 d1c2 b8d7 c4d5 e6d5");
    book_add_line("g1f3 g8f6 c2c4 g7g6 b1c3 f8g7 e2e4 d7d6 d2d4 e8g8 f1e2 e7e5 e1g1 b8a6 f1e1 c7c6 e2f1 c8g4 d4d5 c6d5");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 c7c5 g1e2 c5d4 e3d4 e8g8 a2a3 b4e7 e2f4 d7d6 f1e2 e6e5 d4e5 d6e5");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 e8g8 f1d3 c7c5 g1f3 d7d5 e1g1 c5d4 e3d4 d5c4 d3c4 b7b6 d1b3 b4c3");
    book_add_line("c2c4 g7g6 g2g3 g8f6 f1g2 c7c6 e2e4 d7d5 c4d5 c6d5 e4e5 f6d7 d2d4 d7b6 b1c3 b8c6 g1e2 f8g7 h2h4 c8f5");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 e7e5 d4b5 d7d6 c2c4 f8e7 f1d3 a7a6 b5c3 e7g5 b1a3 g5c1 a1c1 d8g5");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 c1e3 e7e6 g2g4 e6e5 d4f5 g7g6 g4g5 g6f5 e4f5 d6d5");
    book_add_line("d2d4 d7d5 c2c4 b8c6 c4d5 d8d5 e2e3 e7e5 b1c3 f8b4 c1d2 b4c3 b2c3 g8f6 f2f3 e8g8 e3e4 d5d6 d4d5 c6e7");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 c7c5 d4d5 d7d6 b1c3 e6d5 c4d5 g7g6 h2h3 a7a6 a2a4 d8e7 c1f4 b8d7 e2e3 f8g7");
    book_add_line("e2e4 c7c5 g1f3 e7e6 d2d4 c5d4 f3d4 b8c6 b1c3 d8c7 g2g3 a7a6 f1g2 g8f6 e1g1 c6d4 d1d4 f8c5 c1f4 d7d6");
    book_add_line("e2e4 c7c5 b1c3 d7d6 f2f4 g7g6 g1f3 f8g7 f1c4 b8c6 e1g1 e7e6 d2d3 g8e7 d1e1 e8g8 c4b3 c6a5 c1e3 b7b6");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 b7b6 g2g3 c8a6 b2b3 a6b7 f1g2 f8b4 c1d2 a7a5 e1g1 e8g8 d1c2 d7d6 d2c3 d6d5");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 c1g5 e7e6 f2f4 d8b6 a2a3 b8c6 d4b3 f8e7 d1f3 h7h6");
    book_add_line("e2e4 e7e5 b1c3 g8f6 g2g3 f8b4 f1g2 b4c3 b2c3 e8g8 g1e2 d7d5 e4d5 f6d5 e1g1 b8c6 a1b1 d8d6 c3c4 d5f6");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 d1c2 d7d5 c4d5 d8d5 e2e3 c7c5 c1d2 b4c3 d2c3 c5d4 c3d4 b8c6 d4c3 e8g8");
    book_add_line("d2d4 d7d5 c2c4 c7c6 b1c3 g8f6 e2e3 e7e6 g1f3 b8d7 f1d3 d5c4 d3c4 b7b5 c4d3 c8b7 e3e4 b5b4 c3a4 c6c5");
    book_add_line("c2c4 e7e6 b1c3 d7d5 d2d4 g8f6 g1f3 f8e7 d1c2 b8d7 c1g5 h7h6 c4d5 h6g5 d5e6 d7b6 e6f7 e8f8 e1c1 f6d5");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 f1d3 e7e5 d4e2 f8e7 e1g1 e8g8 g1h1 b8d7 e2g3 d7c5");
    book_add_line("d2d4 g8f6 c2c4 g7g6 g2g3 f8g7 f1g2 d7d5 c4d5 f6d5 e2e4 d5b6 g1e2 c7c5 d4d5 e7e6 b1c3 e6d5 e4d5 e8g8");
    book_add_line("g1f3 d7d5 d2d4 g8f6 c2c4 e7e6 b1c3 f8e7 c1g5 e8g8 e2e3 h7h6 g5h4 b7b6 f1d3 c8b7 e1g1 b8d7 c4d5 f6d5");
    book_add_line("d2d4 d7d5 c2c4 e7e6 b1c3 f8e7 c4d5 e6d5 c1f4 c7c6 e2e3 e7d6 f4g3 c8f5 d1b3 d6g3 h2g3 d8b6 f1e2 b8a6");
    book_add_line("e2e4 e7e5 g1f3 b8c6 d2d4 e5d4 f3d4 g8f6 d4c6 b7c6 e4e5 d8e7 d1e2 f6d5 c2c4 d5b6 b1c3 c8b7 c1d2 e8c8");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 f8b4 c1d2 d8e7 g2g3 b8c6 b1c3 b4c3 d2c3 f6e4 a1c1 d7d6 f1g2 e8g8 d4d5 e4c3");
    book_add_line("d2d4 g8f6 c2c4 g7g6 b1c3 f8g7 e2e4 d7d6 f1e2 e8g8 c1g5 b8a6 g1f3 h7h6 g5e3 f6g4 e3c1 e7e5 e1g1 c7c6");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 b7b6 e2e3 c8b7 f1d3 g7g6 b1c3 f8g7 e1g1 d7d6 e3e4 e8g8 f1e1 b8d7 d3f1 a7a6");
    book_add_line("g1f3 g8f6 c2c4 e7e6 g2g3 d7d5 f1g2 d5c4 d1a4 b8d7 a4c4 a7a6 c4c2 a8b8 e1g1 b7b5 a2a4 f8e7 a4b5 a6b5");
    book_add_line("b2b3 e7e5 c1b2 b8c6 e2e3 g8f6 g1f3 f8d6 d2d3 e8g8 b1d2 f8e8 a2a3 b7b5 c2c4 b5b4 a3a4 c8b7 d1c2 d6c5");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 d1c2 e8g8 a2a3 b4c3 c2c3 b7b6 c1g5 c8a6 c3f3 b8c6 e2e3 c6a5 a1c1 c7c5");
    book_add_line("g1f3 d7d5 g2g3 c7c6 f1g2 c8g4 c2c3 b8d7 d1b3 d8b6 d2d3 e7e6 c1e3 f8c5 e3c5 d7c5 b3c2 g8f6 b1d2 a7a5");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 g8f6 d1e2 f8c5 d2d3 c6d4 f3d4 c5d4 c2c3 d4b6 c1g5 e8g8 b1d2 d7d5 b5a4 c7c6");
    book_add_line("d2d4 d7d5 c2c4 c7c6 g1f3 g8f6 b1c3 d5c4 a2a4 c6c5 d4d5 c8f5 e2e3 e7e6 f1c4 e6d5 c3d5 b8c6 d1b3 d8d7");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 a7a6 b5a4 g8f6 e1g1 f8c5 c2c3 b7b5 a4b3 d7d6 a2a4 c8g4 h2h3 g4f3 d1f3 e8g8");
    book_add_line("d2d4 g8f6 g1f3 g7g6 b1c3 d7d5 c1f4 c7c6 d1d2 c8f5 f3e5 f6d7 e1c1 f8g7 f4h6 g7h6 d2h6 d7e5 d4e5 d8c7");
    book_add_line("d2d4 g8f6 g1f3 c7c5 d4d5 b7b5 c1g5 f6e4 g5h4 c8b7 e2e3 g7g6 c2c3 d8a5 b1d2 e4d2 d1d2 f8g7 e3e4 d7d6");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 e7e5 d4b5 d7d6 c1g5 a7a6 b5a3 b7b5 g5f6 g7f6 c3d5 f8g7");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 g7g6 c1e3 f8g7 f2f3 b8c6 d1d2 e8g8 e1c1 d6d5 e4d5 f6d5");
    book_add_line("e2e4 e7e5 f2f4 e5f4 g1f3 g7g5 h2h4 g5g4 f3e5 d7d6 e5g4 g8f6 g4f6 d8f6 b1c3 b8c6 c3d5 f6g6 d2d3 g6g3");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 f1e2 e7e6 e1g1 f8e7 f2f4 e8g8 d1e1 b7b5 e2f3 b5b4");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1c4 f8c5 b2b4 c5b4 c2c3 b4a5 d2d4 e5d4 e1g1 g8e7 f3g5 d7d5 e4d5 c6e5 c4b3 e8g8");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 e8g8 f1d3 d7d5 g1f3 c7c5 e1g1 b8c6 a2a3 b4c3 b2c3 d5c4 d3c4 d8c7");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 e7e6 d4b5 f8b4 a2a3 b4c3 b5c3 d7d5 e4d5 e6d5 f1d3 e8g8");
    book_add_line("e2e4 e7e5 g1f3 g8f6 f3e5 d7d6 e5f3 f6e4 d2d4 d6d5 f1d3 f8d6 e1g1 e8g8 c2c4 c7c6 f1e1 c8f5 d1c2 f5g6");
    book_add_line("g1f3 d7d5 d2d4 g8f6 c2c4 e7e6 b1c3 d5c4 e2e4 f8b4 c1g5 c7c5 f1c4 c5d4 f3d4 b4c3 b2c3 d8a5 c4b5 b8d7");
    book_add_line("e2e4 c7c5 g1f3 d7d6 f1b5 b8c6 e1g1 c8d7 f1e1 g8f6 h2h3 c6e5 a2a4 e5f3 d1f3 a7a6 b5d7 f6d7 b2b3 g7g6");
    book_add_line("d2d4 g8f6 c2c4 g7g6 g1f3 f8g7 g2g3 d7d5 c4d5 f6d5 f1g2 d5b6 e1g1 b8c6 e2e3 e7e5 b1c3 e8g8 d4d5 c6e7");
    book_add_line("d2d4 g8f6 c2c4 g7g6 b1c3 f8g7 e2e4 d7d6 f1d3 e8g8 g1e2 b8c6 e1g1 f6h5 d3c2 e7e5 d4d5 c6e7 a2a4 g8h8");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 g8f6 e1g1 f6e4 d2d4 e4d6 b5c6 d7c6 d4e5 d6f5 d1d8 e8d8 b1c3 d8e8 h2h3 f8e7");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 g8f6 b1c3 c5d4 f3d4 a7a6 c1e3 e7e5 d4b3 c8e6 f2f3 f8e7 d1d2 e8g8 e1c1 b8d7");
    book_add_line("d2d4 d7d5 g1f3 g8f6 c1g5 f6e4 g5h4 c7c5 d4c5 b8c6 e2e3 g7g6 b1d2 e4c5 d2b3 f8g7 c2c3 c5e4 f1d3 e8g8");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 a7a6 b5a4 g8f6 e1g1 f8e7 f1e1 b7b5 a4b3 e8g8 c2c3 d7d5 e4d5 f6d5 f3e5 c6e5");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g2g3 c7c5 g1f3 c5d4 f3d4 d7d5 f1g2 e6e5 d4f3 d5d4 e1g1 b8c6 e2e3 f8c5 e3d4 e5d4");
    book_add_line("g1f3 g8f6 c2c4 b7b6 g2g3 c8b7 f1g2 e7e6 e1g1 f8e7 b1c3 e8g8 f1e1 f6e4 c3e4 b7e4 d2d3 e4b7 d3d4 b7e4");
    book_add_line("e2e4 e7e6 d2d4 d7d5 e4e5 c7c5 c2c3 b8c6 g1f3 c8d7 f1e2 g8e7 b1a3 c5d4 c3d4 e7f5 a3c2 d8b6 g2g4 f5e7");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 e8g8 f1d3 d7d5 g1f3 c7c5 e1g1 d5c4 d3c4 b8d7 a2a3 c5d4 a3b4 d4c3");
    book_add_line("e2e4 e7e5 f2f4 e5f4 g1f3 g7g5 h2h4 g5g4 f3e5 g8f6 d2d4 d7d6 e5d3 b8c6 c2c3 f6e4 c1f4 d6d5 b1d2 f8d6");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 g7g6 c1e3 f8g7 f2f3 b8c6 d1d2 e8g8 e1c1 d6d5 d2e1 e7e5");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 c1e3 e7e6 f2f3 b7b5 g2g4 h7h6 d1d2 b8d7 e1c1 c8b7");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 c1e3 f6g4 e3g5 h7h6 g5h4 g7g5 h4g3 f8g7 h2h3 g4f6");
    book_add_line("d2d4 d7d5 c2c4 b8c6 b1c3 d5c4 g1f3 g8f6 e2e4 c8g4 c1e3 g4f3 g2f3 e7e5 d4d5 c6b8 f1c4 b8d7 d1b3 f8c5");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 g8f6 d2d3 f8c5 c2c3 e8g8 b5c6 b7c6 f3e5 d7d5 c1g5 f8e8 f2f4 d5e4 d3d4 c5b6");
    book_add_line("d2d4 g8f6 c2c4 e7e6 g1f3 f8b4 c1d2 a7a5 g2g3 d7d5 f1g2 d5c4 d1c2 b8c6 c2c4 d8d5 c4d5 e6d5 b1c3 c8g4");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 f2f3 b8c6 e2e4 d7d5 c4d5 e6d5 e4e5 f6g8 f3f4 g8h6 g1f3 c8g4 c1e3 h6f5");
    book_add_line("c2c4 g8f6 g2g3 c7c6 b2b3 d7d5 c1b2 c8g4 f1g2 e7e6 g1f3 b8d7 e1g1 f8d6 d2d3 e8g8 b1d2 a7a5 a2a3 g4h5");
    book_add_line("e2e4 e7e5 f1c4 g8f6 d2d3 c7c6 g1f3 f8e7 e1g1 d7d6 c4b3 b8d7 c2c3 d7f8 f1e1 f8g6 b1d2 e8g8 d3d4 h7h6");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 a7a6 b5a4 g8f6 e1g1 f8c5 c2c3 b7b5 a4c2 d7d5 d2d4 d5e4 b1d2 e4f3 d1f3 c5e7");
    book_add_line("e2e4 c7c5 g1f3 d7d6 d2d4 c5d4 f3d4 g8f6 b1c3 a7a6 c1e3 e7e5 d4f3 f8e7 f1c4 e8g8 d1d3 c8e6 e1c1 e6c4");
    book_add_line("d2d4 g8f6 c2c4 e7e6 b1c3 f8b4 e2e3 e8g8 g1e2 d7d5 a2a3 b4e7 e2g3 c7c5 d4c5 d5c4 d1c2 b8d7 f1c4 e7c5");
    book_add_line("e2e4 e7e5 b1c3 g8f6 g2g3 f8c5 f1g2 c7c6 d1e2 d7d5 d2d3 e8g8 g1f3 b8d7 e1g1 f8e8 f3h4 d5e4 d3e4 g7g6");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 d7d6 c1g5 e7e6 d1d2 f8e7 e1c1 e8g8 f2f4 h7h6 g5h4 c6d4");
    book_add_line("d2d4 g8f6 c1g5 e7e6 e2e3 h7h6 g5h4 d7d6 f1d3 b8d7 c2c4 f8e7 b1c3 b7b6 h4g3 e8g8 h2h3 c8b7 g1f3 c7c5");
    book_add_line("e2e4 d7d6 d2d4 g8f6 b1c3 g7g6 f2f4 f8g7 g1f3 e8g8 f1d3 b8a6 e1g1 c7c5 d4d5 c8g4 a2a3 a6c7 d1e1 g4f3");
    book_add_line("c2c4 e7e6 d2d4 d7d5 b1c3 f8b4 d1a4 b8c6 e2e3 g8f6 a2a3 b4c3 b2c3 e8g8 g1f3 c8d7 a4c2 c6a5 c4d5 e6d5");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 a7a6 b5a4 g8f6 e1g1 f6e4 d2d4 b7b5 a4b3 d7d5 d4e5 c8e6 c1e3 f8e7 c2c3 d8d7");
    book_add_line("c2c4 c7c5 g1f3 g8f6 b1c3 b8c6 d2d4 c5d4 f3d4 e7e6 a2a3 f8e7 e2e4 e8g8 d4f3 d7d6 f1e2 b7b6 e1g1 c8b7");
    book_add_line("e2e4 e7e5 g1f3 b8c6 f1b5 a7a6 b5a4 g8f6 e1g1 f6e4 d2d4 b7b5 a4b3 d7d5 d4e5 c8e6 c1e3 e4c5 b1c3 c5b3");
    book_add_line("e2e4 c7c5 g1f3 b8c6 d2d4 c5d4 f3d4 g8f6 b1c3 e7e6 d4b5 f8b4 a2a3 b4c3 b5c3 d7d5 f1d3 d5d4 c3e2 e6e5");
    book_add_line("g1f3 g8f6 g2g3 d7d5 f1g2 c7c6 e1g1 c8g4 d2d3 b8d7 c1f4 f6h5 f4c1 h5f6 h2h3 g4h5 c1f4 e7e6 c2c4 f8e7");
}

//end
