/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  Initialization of several data structure used in the program.
//-------------------------------------------------------------------------------------------------

U64     data_gen_moves(int rf, int ff, int ri, int fi, int single);

//-------------------------------------------------------------------------------------------------
//  Initialize data.
//-------------------------------------------------------------------------------------------------
void data_init(void) 
{
	int     i, s;
	char    r, f, p, rm, fm;
	char    rt, ft, rc, fc;
    int     mvpc;
    int     cppc;
    int     sort;

	for (s = 0; s < 64; s++)
    {
		bb_square[s] = 0;
		bb_set(bb_square[s], s);
	}

	piece_value[EMPTY]  = 0;
	piece_value[PAWN]   = VALUE_PAWN;
	piece_value[KNIGHT] = VALUE_KNIGHT;
	piece_value[BISHOP] = VALUE_BISHOP;
	piece_value[ROOK]   = VALUE_ROOK;
	piece_value[QUEEN]  = VALUE_QUEEN;
	piece_value[KING]   = VALUE_KING;

   	see_value[EMPTY]  = 0;
	see_value[PAWN]   = 100;
	see_value[KNIGHT] = 300;
	see_value[BISHOP] = 300;
	see_value[ROOK]   = 500;
	see_value[QUEEN]  = 900;
	see_value[KING]   = 1000;

    for (mvpc = EMPTY; mvpc <= KING; mvpc++)
    {
        for (cppc = EMPTY; cppc <= KING; cppc++)
        {
            if (see_value[cppc] > see_value[mvpc])
                sort = 3000;
            else
                if (see_value[cppc] == see_value[mvpc])
                    sort = 2000;
                else
                    sort = 1000;
            capture_value[mvpc][cppc] = see_value[cppc] * sort + (see_value[KING] - see_value[mvpc]);
        }
    }
    
    for (mvpc = EMPTY; mvpc <= KING; mvpc++)
    {
        promotion_value[mvpc] = 3000 * see_value[mvpc];

        for (cppc = EMPTY; cppc <= KING; cppc++)
        {
            prom_capt_value[mvpc][cppc] = see_value[mvpc] * 3000 + see_value[cppc];
        }
    }

    is_sliding[EMPTY]  = 0;
	is_sliding[PAWN]   = 0;
	is_sliding[KING]   = 0;
	is_sliding[KNIGHT] = 0;
	is_sliding[BISHOP] = 1;
	is_sliding[ROOK]   = 1;
	is_sliding[QUEEN]  = 1;

	piece_letter[PAWN]   = 'P';
	piece_letter[KING]   = 'K';
	piece_letter[QUEEN]  = 'Q';
	piece_letter[ROOK]   = 'R';
	piece_letter[BISHOP] = 'B';
	piece_letter[KNIGHT] = 'N';

	promo_letter[QUEEN]  = 'q';
	promo_letter[ROOK]   = 'r';
	promo_letter[BISHOP] = 'b';
	promo_letter[KNIGHT] = 'n';

	// king moves
	for (r = 0; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			king_moves[r * 8 + f] = 0;
			king_moves[r * 8 + f] |= data_gen_moves(r, f, +1,  0, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f, -1,  0, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f,  0, +1, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f,  0, -1, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f, +1, +1, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f, +1, -1, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f, -1, +1, 1);
			king_moves[r * 8 + f] |= data_gen_moves(r, f, -1, -1, 1);
		}
    }

	// knight moves
	for (r = 0; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			knight_moves[r * 8 + f] = 0;
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, +2, +1, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, +2, -1, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, -2, +1, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, -2, -1, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, +1, +2, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, -1, +2, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, +1, -2, 1);
			knight_moves[r * 8 + f] |= data_gen_moves(r, f, -1, -2, 1);
		}
    }

	// from to path
	memset(&from_to_path, 0, sizeof(from_to_path));

	for (r = 0; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			// east
			for (ft = f + 2; ft < 8; ft++)
            {
				for (fm = f + 1; fm < ft; fm++)
                {
					bb_set_rf(from_to_path[r * 8 + f][r * 8 + ft], r, fm);
				}
            }
			// west
			for (ft = f - 2; ft >= 0; ft--)
            {
				for (fm = f - 1; fm > ft; fm--)
                {
					bb_set_rf(from_to_path[r * 8 + f][r * 8 + ft], r, fm);
				}
            }
			// south
			for (rt = r + 2; rt < 8; rt++)
            {
				for (rm = r + 1; rm < rt; rm++)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + f], rm, f);
				}
            }
			// north
			for (rt = r - 2; rt >= 0; rt--) 
            {
				for (rm = r - 1; rm > rt; rm--)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + f], rm, f);
				}
            }
			// southeast
			rt = r + 2;
			ft = f + 2;
			while(rt < 8 && ft < 8)
            {
				rm = r + 1;
				fm = f + 1;
				while (rm < rt && fm < ft)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + ft], rm, fm);
					rm++;
					fm++;
				}
				rt++;
				ft++;
			}
			// northeast
			rt = r - 2;
			ft = f + 2;
			while(rt >= 0 && ft < 8)
            {
				rm = r - 1;
				fm = f + 1;
				while (rm > rt && fm < ft)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + ft], rm, fm);
					rm--;
					fm++;
				}
				rt--;
				ft++;
			}
			// southwest
			rt = r + 2;
			ft = f - 2;
			while(rt < 8 && ft >= 0)
            {
				rm = r + 1;
				fm = f - 1;
				while (rm < rt && fm > ft)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + ft], rm, fm);
					rm++;
					fm--;
				}
				rt++;
				ft--;
			}
			// northwest
			rt = r - 2;
			ft = f - 2;
			while(rt >= 0 && ft >= 0)
            {
				rm = r - 1;
				fm = f - 1;
				while (rm > rt && fm > ft)
                {
					bb_set_rf(from_to_path[r * 8 + f][rt * 8 + ft], rm, fm);
					rm--;
					fm--;
				}
				rt--;
				ft--;
			}
		}
    }
        
	// rook and queen - rank and file moves.
	memset(&rankfile_east, 0, sizeof(rankfile_east));
	memset(&rankfile_west, 0, sizeof(rankfile_east));
	memset(&rankfile_south, 0, sizeof(rankfile_east));
	memset(&rankfile_north, 0, sizeof(rankfile_east));

	for (r = 0; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			rankfile_east[r * 8 + f] = data_gen_moves(r, f, 0, 1, 0);
			rankfile_south[r * 8 + f] = data_gen_moves(r, f, 1, 0, 0);
			rankfile_west[r * 8 + f] = data_gen_moves(r, f, 0, -1, 0);
			rankfile_north[r * 8 + f] = data_gen_moves(r, f, -1, 0, 0);
		}
	}

	// bishop and queen - diagonal moves.
	memset(&diagonal_sw, 0, sizeof(diagonal_sw));
	memset(&diagonal_se, 0, sizeof(diagonal_se));
	memset(&diagonal_nw, 0, sizeof(diagonal_nw));
	memset(&diagonal_ne, 0, sizeof(diagonal_ne));

	for (r = 0; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			diagonal_se[r * 8 + f] = data_gen_moves(r, f, 1, 1, 0);
			diagonal_ne[r * 8 + f] = data_gen_moves(r, f, -1, 1, 0);
			diagonal_sw[r * 8 + f] = data_gen_moves(r, f, 1, -1, 0);
			diagonal_nw[r * 8 + f] = data_gen_moves(r, f, -1, -1, 0);
		}
	}

	// rank file position
	p = 0;
	for (rc = '8'; rc >= '1'; rc--)
    {
		for (fc = 'a'; fc <= 'h'; fc++)
        {
			rank_file_notation[p][0] = fc;
			rank_file_notation[p][1] = rc;
			rank_file_notation[p][2] = 0;
			p++;
		}
	}

    // Pawn attack squares
	for (r = 0; r < 6; r++)
    {
		for (f = 0; f < 8; f++)
        {
			if (f == 0)
				bb_set_rf(pawn_attack[WHITE][r * 8 + f], r + 1, f + 1);
			if (f >= 1 && f <= 6)
            {
				bb_set_rf(pawn_attack[WHITE][r * 8 + f], r + 1, f + 1);
				bb_set_rf(pawn_attack[WHITE][r * 8 + f], r + 1, f - 1);
			}
			if (f == 7)
				bb_set_rf(pawn_attack[WHITE][r * 8 + f], r + 1, f - 1);
		}
    }

	for (r = 1; r < 8; r++)
    {
		for (f = 0; f < 8; f++)
        {
			if (f == 0)
				bb_set_rf(pawn_attack[BLACK][r * 8 + f], r - 1, f + 1);
			if (f >= 1 && f <= 6) {
				bb_set_rf(pawn_attack[BLACK][r * 8 + f], r - 1, f + 1);
				bb_set_rf(pawn_attack[BLACK][r * 8 + f], r - 1, f - 1);
			}
			if (f == 7)
				bb_set_rf(pawn_attack[BLACK][r * 8 + f], r - 1, f - 1);
		}
    }

	// path to calculate check attacks
	for (i = 0; i < 64; i++)
    {
		rankfile_rays[i] = 0;
		rankfile_rays[i] |= rankfile_north[i];
		rankfile_rays[i] |= rankfile_south[i];
		rankfile_rays[i] |= rankfile_east[i];
		rankfile_rays[i] |= rankfile_west[i];
		diagonal_rays[i] = 0;
		diagonal_rays[i] |= diagonal_ne[i];
		diagonal_rays[i] |= diagonal_nw[i];
		diagonal_rays[i] |= diagonal_se[i];
		diagonal_rays[i] |= diagonal_sw[i];
	}
}

//-------------------------------------------------------------------------------------------------
//  Set the bits to define a path in the board from a starting location.
//-------------------------------------------------------------------------------------------------
U64 data_gen_moves(int rf, int ff, int ri, int fi, int single)
{
	int     rt, ft;
	U64     b = 0;

	rt = rf + ri;
	ft = ff + fi;
	while (rt >= 0 && rt < 8 && ft >= 0 && ft < 8)
    {
		bb_set_rf(b, rt, ft);
		if (single)
			break;
		rt += ri;
		ft += fi;
	}
	return b;
}

//END
