/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//------------------------------------------------------------------------------------
//  Initialization of eval data.
//------------------------------------------------------------------------------------
void eval_init(void)
{
    int     r, f, rp, fp;
    int     sq;

    ep_on = 0;

    // Files
    for (f = 0; f < 8; f++)
    {
        bb_files[f] = 0;
        for (r = 0; r < 8; r++)
        {
            bb_set_rf(bb_files[f], r, f);
        }
    }

    // Passed Pawn mask
    memset(bb_pawn_pass, 0, sizeof(bb_pawn_pass));
    for (r = 0; r < 8; r++) 
    {
        for (f = 0; f < 8; f++)
        {
            // White
            rp = r - 1;
            fp = f;
            while (rp >= 0) 
            {
                bb_set_rf(bb_pawn_pass[WHITE][r * 8 + f], rp, fp);
                if (fp > 0)
                    bb_set_rf(bb_pawn_pass[WHITE][r * 8 + f], rp, fp - 1);
                if (fp < 7)
                    bb_set_rf(bb_pawn_pass[WHITE][r * 8 + f], rp, fp + 1);
                rp--;
            }
            // Black
            rp = r + 1;
            fp = f;
            while (rp < 8)
            {
                bb_set_rf(bb_pawn_pass[BLACK][r * 8 + f], rp, fp);
                if (fp > 0)
                    bb_set_rf(bb_pawn_pass[BLACK][r * 8 + f], rp, fp - 1);
                if (fp < 7)
                    bb_set_rf(bb_pawn_pass[BLACK][r * 8 + f], rp, fp + 1);
                rp++;
            }
        }
    }

    // Weak Pawn Mask
    memset(bb_pawn_weak, 0, sizeof(bb_pawn_weak));
    for (sq = 0; sq < 64; sq++)
    {
        f = square_file(sq);
        r = square_rank(sq);
        if (r > 0 && r < 7)
        {
            if (f > 0)
                bb_pawn_weak[WHITE][sq] |= rankfile_south[sq - 9];
            if (f < 7)
                bb_pawn_weak[WHITE][sq] |= rankfile_south[sq - 7];

            if (f > 0)
                bb_pawn_weak[BLACK][sq] |= rankfile_north[sq + 7];
            if (f < 7)
                bb_pawn_weak[BLACK][sq] |= rankfile_north[sq + 9];
        }
    }

	// Connected pawn mask
	memset(bb_pawn_conn, 0, sizeof(bb_pawn_conn));
	for (sq = 8; sq < 56; sq++)
	{
		if (square_file(sq) > 0)
		{
			bb_set(bb_pawn_conn[sq], sq - 1);
			bb_set(bb_pawn_conn[sq], sq - 9);
			bb_set(bb_pawn_conn[sq], sq + 7);
		}
		if (square_file(sq) < 7)
		{
			bb_set(bb_pawn_conn[sq], sq + 1);
			bb_set(bb_pawn_conn[sq], sq - 7);
			bb_set(bb_pawn_conn[sq], sq + 9);
		}
	}

    // Pawn support mask
    memset(bb_pawn_supp, 0, sizeof(bb_pawn_supp));
    for (sq = 0; sq < 64; sq++)
    {
        f = square_file(sq);
        r = square_rank(sq);
        if (r > 0 && r < 7)
        {
            if (f > 0)
                bb_pawn_supp[WHITE][sq] |= bb_square[sq - 1] | bb_square[sq + 7];
            if (f < 7)
                bb_pawn_supp[WHITE][sq] |= bb_square[sq + 1] | bb_square[sq + 9];

            if (f > 0)
                bb_pawn_supp[BLACK][sq] |= bb_square[sq - 1] | bb_square[sq - 9];
            if (f < 7)
                bb_pawn_supp[BLACK][sq] |= bb_square[sq + 1] | bb_square[sq - 7];
        }
    }

    // Isolated pawn mask
    memset(bb_pawn_isol, 0, sizeof(bb_pawn_isol));
    for (sq = 0; sq < 64; sq++)
    {
        f = square_file(sq);
        if (f > 0)
        {
            for (r = 1 ; r < 7; r++)
                bb_set_rf(bb_pawn_isol[sq], r, f - 1);
        }
        if (f < 7)
        {
            for (r = 1 ; r < 7; r++)
                bb_set_rf(bb_pawn_isol[sq], r, f + 1);
        }
    }

    // Pawn capture mask
    memset(bb_pawn_capt, 0, sizeof(bb_pawn_capt));
    for (sq = 0; sq < 64; sq++)
    {
        f = square_file(sq);
        r = square_rank(sq);
        if (r > 0 && r < 7)
        {
            if (f > 0)
            {
                bb_set_rf(bb_pawn_capt[WHITE][sq], r - 1, f - 1);
                bb_set_rf(bb_pawn_capt[BLACK][sq], r + 1, f - 1);
            }
            if (f < 7)
            {
                bb_set_rf(bb_pawn_capt[WHITE][sq], r - 1, f + 1);
                bb_set_rf(bb_pawn_capt[BLACK][sq], r + 1, f + 1);
            }
        }
    }
}

//END