/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//------------------------------------------------------------------------------------
//  Initialization of piece square table data.
//------------------------------------------------------------------------------------

#define MS(op, eg)	MAKE_SCORE(op, eg)

// pawn
int PST_P_OP[64] =
{
	-10,  -5,   0,   5,   5,   0,  -5, -10,
	-10,  -5,   0,   5,   5,   0,  -5, -10,
	-10,  -5,   0,   5,   5,   0,  -5, -10,
	-10,  -5,   0,  15,  15,   0,  -5, -10,
	-10,  -5,   0,  20,  20,   0,  -5, -10,
	-10,  -5,   0,  10,  10,   0,  -5, -10,
	-10,  -5,   0,   5,   5,   0,  -5, -10,
	-10,  -5,   0,   5,   5,   0,  -5, -10
};
int PST_P_EG[64] =
{
	 0,   0,   0,   0,   0,   0,   0,   0,
	15,  15,  15,  15,  15,  15,  15,  15,
	10,  10,  10,  10,  10,  10,  10,  10,
	 8,   8,   8,   8,   8,   8,   8,   8,
	 4,   4,   4,   4,   4,   4,   4,   4,
	 0,   0,   0,   0,   0,   0,   0,   0,
     0,   0,   0,   0,   0,   0,   0,   0,
     0,   0,   0,   0,   0,   0,   0,   0
};
// king
int PST_K_OP[64] = 
{
	-40, -40, -40, -40, -40, -40, -40, -40,
	-35, -35, -35, -35, -35, -35, -35, -35,
	-30, -30, -30, -30, -30, -30, -30, -30,
	-25, -25, -25, -25, -25, -25, -25, -25,
	-20, -20, -20, -20, -20, -20, -20, -20,
	-15, -15, -15, -15, -15, -15, -15, -15,
	-10, -10, -10, -10, -10, -10, -10, -10,
	  0,  10,  10, -10, -10,  10,  10,   0
};
int PST_K_EG[64] = 
{
	-10, -10, -10, -10, -10, -10, -10,  -10,
	-10,  20,  30,  40,  40,  30,  20,  -10,
	-10,  30,  40,  50,  50,  40,  30,  -10,
	-10,  40,  50,  60,  60,  50,  40,  -10,
	-10,  40,  50,  60,  60,  50,  40,  -10,
	-10,  30,  40,  50,  50,  40,  30,  -10,
	-10,  20,  30,  40,  40,  30,  20,  -10,
	-10, -10, -10, -10, -10, -10, -10,  -10
};
int PST_K_EG_KS[64] = 
{
	-10, -10, -10, -10, -10, -10, -10,  -10,
	-10,   0,   5,  10,  20,  30,  40,  -10,
	-10,   0,   5,  10,  20,  30,  40,  -10,
	-10,   0,   5,  10,  20,  30,  40,  -10,
	-10,   0,   5,  10,  20,  30,  40,  -10,
	-10,   0,   5,  10,  20,  20,  30,  -10,
	-10,   0,   5,  10,  10,  20,  20,  -10,
	-10, -10, -10, -10, -10, -10, -10,  -10
};
int PST_K_EG_QS[64] = 
{
	-10, -10, -10, -10, -10, -10, -10,  -10,
	-10,  40,  30,  20,  10,   5,   0,  -10,
	-10,  40,  30,  20,  10,   5,   0,  -10,
	-10,  40,  30,  20,  10,   5,   0,  -10,
	-10,  40,  30,  20,  10,   5,   0,  -10,
	-10,  30,  20,  20,  10,   5,   0,  -10,
	-10,  20,  20,  10,  10,   5,   0,  -10,
	-10, -10, -10, -10, -10, -10, -10,  -10
};
// knight
int PST_N_OP[64] =
{
    -50,  -8,  -4,  -4,  -4,  -4,  -8, -50,
     -8,  -4,  10,  10,  10,  10,  -4,  -8,
     -4,  10,  10,  10,  10,  10,  10,  -4,
     -4,  10,  10,  18,  18,  10,  10,  -4,
     -4,  10,  10,  18,  18,  10,  10,  -4,
     -4,  10,  10,  15,  15,  10,  10,  -4,
     -8,  -4,  10,  10,  10,  10,  -4,  -8,
    -12,  -8,  -4,  -4,  -4,  -4,  -8, -12
};
int PST_N_EG[64] =
{
    -50,  -8,  -4,  -4,  -4,  -4,  -8, -50,
     -8,  -4,   8,   8,   8,   8,  -4,  -8,
     -4,   8,  10,  10,  10,  10,   8,  -4,
     -4,   8,  10,  18,  18,  10,   8,  -4,
     -4,   8,  10,  18,  18,  10,   8,  -4,
     -4,   8,  10,  15,  15,  10,   8,  -4,
     -8,  -4,   5,   5,   5,   5,  -4,  -8,
    -12,  -8,  -4,  -4,  -4,  -4,  -8, -12
};
// bishop
int PST_B_OP[64] =
{
	-4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
	-4,   5,   5,   5,   5,   5,   5,  -4,
	-4,   5,   6,   6,   6,   6,   5,  -4,
	-4,   5,   6,   8,   8,   6,   5,  -4,
	-4,   5,   6,   8,   8,   6,   5,  -4,
	-4,   5,   6,   6,   6,   6,   5,  -4,
	-4,   6,   5,   5,   5,   5,   6,  -4,
	-4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
};
int PST_B_EG[64] =
{
	-4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
	-4,   5,   5,   5,   5,   5,   5,  -4,
	-4,   5,   6,   6,   6,   6,   5,  -4,
	-4,   5,   6,   8,   8,   6,   5,  -4,
	-4,   5,   6,   8,   8,   6,   5,  -4,
	-4,   5,   6,   6,   6,   6,   5,  -4,
	-4,   6,   5,   5,   5,   5,   6,  -4,
	-4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
};
// rook
int PST_R_OP[64] =
{
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
	1,   2,   3,   4,   4,   3,   2,   1,
};
int PST_R_EG[64] =
{
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0
};
// queen
int PST_Q_OP[64] =
{
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
	0,   0,   0,   0,   0,   0,   0,   0,
   -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4
};
int PST_Q_EG[64] =
{
    -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4,
    -4,  10,  10,  10,  10,  10,  10,  -4,
    -4,  10,  15,  16,  16,  15,  10,  -4,
    -4,  10,  16,  20,  20,  16,  10,  -4,
    -4,  10,  16,  20,  20,  16,  10,  -4,
    -4,  10,  15,  16,  16,  15,  10,  -4,
    -4,  10,  10,  10,  10,  10,  10,  -4,
    -4,  -4,  -4,  -4,  -4,  -4,  -4,  -4
};

void    eval_pst_assign(int pst[2][64], int op[64], int eg[64], int sq);

//------------------------------------------------------------------------------------
//  Initialization of piece square table data.
//------------------------------------------------------------------------------------
void eval_pst_init(void)
{
    int     sq;

	for (sq = 0; sq < 64; sq++)
	{
        eval_pst_assign(pst_pawn,    PST_P_OP, PST_P_EG,    sq);
        eval_pst_assign(pst_knight,  PST_N_OP, PST_N_EG,    sq);
        eval_pst_assign(pst_bishop,  PST_B_OP, PST_B_EG,    sq);
        eval_pst_assign(pst_rook,    PST_R_OP, PST_R_EG,    sq);
        eval_pst_assign(pst_queen,   PST_Q_OP, PST_Q_EG,    sq);
        eval_pst_assign(pst_king,    PST_K_OP, PST_K_EG,    sq);
        eval_pst_assign(pst_king_ks, PST_K_OP, PST_K_EG_KS, sq);
        eval_pst_assign(pst_king_qs, PST_K_OP, PST_K_EG_QS, sq);
	}

}

//------------------------------------------------------------------------------------
//  Assign corresponding sq value to white and black pst.
//------------------------------------------------------------------------------------
void eval_pst_assign(int pst[2][64], int op[64], int eg[64], int sq)
{
    pst[WHITE][sq] = MAKE_SCORE(op[sq], eg[sq]);
	pst[BLACK][sq] = MAKE_SCORE(op[sq ^ 070], eg[sq ^ 070]);
}

//END
