/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  History and killer heuristic
//-------------------------------------------------------------------------------------------------

int		hist_tot[COLORS][NUM_PIECES][64];
int		hist_hit[COLORS][NUM_PIECES][64];
int		hist_val[COLORS][NUM_PIECES][64];

//-------------------------------------------------------------------------------------------------
//  Inititalize tables.
//-------------------------------------------------------------------------------------------------
void move_hist_init(void)
{
	memset(hist_tot, 0, sizeof(hist_tot));
	memset(hist_hit, 0, sizeof(hist_hit));
	memset(hist_val, 0, sizeof(hist_val));
}

//-------------------------------------------------------------------------------------------------
//  Update history table and killer move list for quiet moves.
//-------------------------------------------------------------------------------------------------
void move_hist_save(int color, int ply, MOVE move, int depth)
{
    int     mvpc;
    int     tosq;

    assert(is_valid(move));

    mvpc = MVPC(move);
    tosq = TOSQ(move);

    hist_tot[color][mvpc][tosq] += 1;
    hist_hit[color][mvpc][tosq] += 1;
    hist_val[color][mvpc][tosq] += (depth * depth);

    if (move != position[ply].killer[color][0])
    {
        position[ply].killer[color][2] = position[ply].killer[color][1];
        position[ply].killer[color][1] = position[ply].killer[color][0];
        position[ply].killer[color][0] = move;
    }
}

//-------------------------------------------------------------------------------------------------
//  Update history for moves that failed low. Will indicate bad history.
//-------------------------------------------------------------------------------------------------
void move_hist_bad(int color, MOVE list[], int size)
{
    int     i;

    for (i = 0; i < size; i++)
        hist_tot[color][MVPC(list[i])][TOSQ(list[i])] += 1;
}

//-------------------------------------------------------------------------------------------------
//  Indicate if move is in the "killer moves" list.
//-------------------------------------------------------------------------------------------------
int move_is_killer(int color, int ply, MOVE move)
{
    return (move == position[ply].killer[color][0] ||
            move == position[ply].killer[color][1] ||
            move == position[ply].killer[color][2]);
}

//-------------------------------------------------------------------------------------------------
//  History value to be used at move ordering.
//-------------------------------------------------------------------------------------------------
int move_hist_val(int color, MOVE move)
{
    int mvpc = MVPC(move);
    int tosq = TOSQ(move);

    if (hist_tot[color][mvpc][tosq] == 0)
        return 0;
    return hist_hit[color][mvpc][tosq] * 100 / hist_tot[color][mvpc][tosq];
}

//-------------------------------------------------------------------------------------------------
//  Indicate if move had history percentage.
//-------------------------------------------------------------------------------------------------
int move_has_bad_hist(int color, MOVE move)
{
    int mvpc = MVPC(move);
    int tosq = TOSQ(move);

    if (hist_tot[color][mvpc][tosq] == 0)
        return 0;

    return hist_hit[color][mvpc][tosq] * 100 / hist_tot[color][mvpc][tosq] < 60 ? 1 : 0;
}

// end
