/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  Move utilities.
//-------------------------------------------------------------------------------------------------

UINT        promo_rank_to[2] = {RANK8, RANK1};

//-------------------------------------------------------------------------------------------------
//  Bad capture moves according SEE score.
//-------------------------------------------------------------------------------------------------
int is_badcap(MOVE move)
{
    if (TYPE(move) != MT_CAPPC)
        return 0;
    if (see_value[CPPC(move)] < see_value[MVPC(move)] && see_capture(move) < 0)
        return 1;
    else
        return 0;
}

//-------------------------------------------------------------------------------------------------
//  Verify if move still valid for current position. Usually moves from transposition table.
//-------------------------------------------------------------------------------------------------
int is_valid(MOVE move)
{
    UINT	mvpc = MVPC(move);
    UINT	frsq = FRSQ(move);
    UINT	tosq = TOSQ(move);
    UINT	type = TYPE(move);

    assert(frsq != tosq);

    //  moving piece is not on square
    if (mvpc != STM_STATE.square[frsq])
        return 0;
    //  pawn should be moving forward
    if (mvpc == PAWN)
    {
        if (board.side_on_move == WHITE)
        {
            if (square_rank(frsq) < square_rank(tosq))
                return 0;
        }
        else
        {
            if (square_rank(frsq) > square_rank(tosq))
                return 0;
        }
    }

    //  Specific type validation
	switch(type) 
	{
	case MT_QUIET:
        if (bb_is_one(board.occupied_squares, tosq))
            return 0;
        if (is_sliding[STM_STATE.square[frsq]])
            return (from_to_path[frsq][tosq] & board.occupied_squares ? 0 : 1);
        return 1;
	case MT_CAPPC:
        if (OPP_STATE.square[tosq] != CPPC(move))
            return 0;
        if (is_sliding[STM_STATE.square[frsq]])
            return (from_to_path[frsq][tosq] & board.occupied_squares ? 0 : 1);
        return 1;
	case MT_EPCAP:
        if (bb_square[tosq] != board.ep_square.u64)
            return 0;
        if (OPP_STATE.square[PWSQ(move)] != PAWN)
            return 0;
        return 1;
	case MT_PAWN2:
        if (bb_is_one(board.occupied_squares, tosq))
            return 0;
        return (from_to_path[frsq][tosq] & board.occupied_squares ? 0 : 1);
	case MT_PROMO:
        if (bb_is_one(board.occupied_squares, tosq))
            return 0;
        if (square_rank(tosq) != promo_rank_to[board.side_on_move])
            return 0;
        return 1;
	case MT_CPPRM:
        if (OPP_STATE.square[tosq] != CPPC(move))
            return 0;
        if (square_rank(tosq) != promo_rank_to[board.side_on_move])
            return 0;
        return 1;
	case MT_CSWKS:
        if (!STM_STATE.can_castle_ks)
            return 0;
        if (from_to_path[E1][H1] & board.occupied_squares)
            return 0;
        if (STM_STATE.square[H1] != ROOK)
            return 0;
        return 1;
	case MT_CSWQS:
        if (!STM_STATE.can_castle_qs)
            return 0;
        if (from_to_path[E1][A1] & board.occupied_squares)
            return 0;
        if (STM_STATE.square[A1] != ROOK)
            return 0;
        return 1;
	case MT_CSBKS:
        if (!STM_STATE.can_castle_ks)
            return 0;
        if (from_to_path[E8][H8] & board.occupied_squares)
            return 0;
        if (STM_STATE.square[H8] != ROOK)
            return 0;
        return 1;
	case MT_CSBQS:
        if (!STM_STATE.can_castle_qs)
            return 0;
        if (from_to_path[E8][A8] & board.occupied_squares)
            return 0;
        if (STM_STATE.square[A8] != ROOK)
            return 0;
        return 1;
	}

	return 1;
}

// end
