/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  Search when in check.
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//  Search
//-------------------------------------------------------------------------------------------------
int search_evasion(int beta, int depth)
{
    MOVES   ml;
	UINT    skip_null  = 0;
    MOVE    hash_move  = MOVE_NONE;
    int     best_score = -INFINITY;
    int     total_move_count = 0;
    int     quiet_move_count = 0;
	int     score = 0;
	UINT    new_incheck;
	int     extensions;
    int     ply = board.ply;
    int     som = board.side_on_move;
    MOVE    move;
    MOVE    searched[MAX_MOVE];

	assert(incheck == 0 || incheck == 1);
	assert(beta >= -INFINITY && beta <= INFINITY);
	assert(depth <= MAX_DEPTH);
    //assert(alpha == beta - 1);

    if (depth <= 0)
		return quiesce(1, beta - 1, beta, 0);

	search.nodes++;

    if (is_draw())
		return 0;

    check_time();

    assert(board.ply >= 0 && board.ply < MAX_PLY);

	position[ply].pv_size = ply;

	// transposition table score or move hint
    if (tt_probe(depth, beta - 1, beta, &score, &hash_move, &skip_null))
        return score;

    assert(score >= -INFINITY && score <= INFINITY);
    assert(skip_null == 0 || skip_null == 1);
    
    moves_init(&ml, 1, hash_move, 0);
    while((move = moves_next(&ml)) != MOVE_NONE)
    {
        assert(is_valid(move));

		// extension if move puts opponent in check
        if (is_check(move))
        {
            new_incheck = 1;
            if (depth < 12)
                extensions = 1;
            else
                extensions = 0;
        }
        else
        {
            new_incheck = 0;
            extensions = 0;
        }

	     // Make move.
		make_move(move);
		if (is_illegal(move))
        {
			undo_move(); 
			continue;
		}
        assert(board.key == zk_board_key());
        assert(board.pawn_key == zk_pawn_key());

		score = -search_zw(new_incheck, 1 - beta, depth - 1 + extensions, 1);
		if (score >= beta && extensions < 0)
		{
			score = -search_zw(new_incheck, 1 - beta, depth - 1, 1);
		}

        undo_move();

        // move count update
        total_move_count++;
		if (IS_QUIET(move))
            searched[quiet_move_count++] = move;

        // score
        if (score > best_score)
        {
            if (score >= beta)
            {
                if (IS_QUIET(move))
                {
                    move_hist_save(som, ply, move, depth);
                    move_hist_bad(som, searched, quiet_move_count - 1);
                }
                tt_save(depth, score, TT_LOWER, move);
                return score;
            }
            best_score = score;
        }
	}

	if (total_move_count == 0)
		return -MATE_VALUE + ply;

    tt_save(depth, best_score, TT_UPPER, MOVE_NONE);

	return best_score;
}

// end
