#include "globals.h"

int get_game(FILE *pgn, char *pgn_moves);
int read_game(FILE *pgn, char *pgn_moves);
int pgn_is_end_game(char *pgn_moves);
int pgn_is_number_or_space(char c1);
MOVE pgn_get_curr_move(char *pgn_move);
void util_pgn_desc(MOVE move, char *string, int inc_file, int inc_rank);

#define MAXMOV 102400

void test_moves(void)
{
	FILE    *pgn = fopen("games01.pgn", "r");
	char    pgn_moves[MAXMOV];
	char    pgn_move[100];
	int     pl;
	int     pm;
    MOVE    curr_move;
    int     game_count = 0;
    int     total_move_count = 0;
    int     total_correct_count = 0;
    MOVE    search_move;
    int     move_count;
    int     piece_moves[7] = {0, 0, 0, 0, 0, 0, 0};
    int     piece_count[7] = {0, 0, 0, 0, 0, 0, 0};
    //int     sq;

    //for (sq = 0; sq < 64; sq++)
    //    {
    //        pst_k_op[WHITE][sq] *= 2;
    //        pst_k_eg[WHITE][sq] *= 2;
    //        pst_k_op[BLACK][sq] *= 2;
    //        pst_k_eg[BLACK][sq] *= 2;
    //    }


	while(read_game(pgn, pgn_moves))
	{
        assert(strlen(pgn_moves) < MAXMOV);
		//printf("GAME: [%s]\n", pgn_moves);

        printf("GAME=[%s]\n", pgn_moves);

        new_game(FEN_NEW_GAME);
        game_count++;
        //if (game_count == 100)
        //    break;
        move_count = 0;
        printf("game: %d\n", game_count);

		pl = 0;
		while(!pgn_is_end_game(&pgn_moves[pl]))
		{
            assert(strlen(pgn_moves) < MAXMOV);
            board.ply = 0;

			while(pgn_is_number_or_space(pgn_moves[pl]))
				pl++;
			pm = 0;
			while(pgn_moves[pl] && pgn_moves[pl] != ' ' && pm < 100)
            {
                if (pgn_moves[pl] == '#' || pgn_moves[pl] == '+')
                {
                    pl++;
                    continue;
                }
				pgn_move[pm++] = pgn_moves[pl++];
            }
			pgn_move[pm] = 0;

            total_move_count++;
            move_count++;
			curr_move = pgn_get_curr_move(pgn_move);

            if (curr_move == MOVE_NONE)
            {
                printf("\n\nNOT FOUND: %s\n\n", pgn_move);
                print_current_moves();
                board_print("move");
                break;
            }
            assert(is_valid(curr_move));

            if (move_count < 21)
            {
                total_correct_count++;
            }
            else
            {
                printf("%3d) PGN_MOVE=", move_count);
                util_print_move(curr_move, 0);

                // search move;
                think(POST_NONE, &search_move, 0);
                piece_moves[MVPC(curr_move)]++;

                if (search_move == curr_move)
                    total_correct_count++;
                else
                    piece_count[MVPC(curr_move)]++;

                printf(" FOUND_MOVE=");
                util_print_move(search_move, 0);
                printf(" %d/%d %3.2f %s\n", total_correct_count, total_move_count,
                    (double)((double)total_correct_count / (double)total_move_count * 100),
                    (search_move == curr_move ? "" : "*"));
            }

            //if (search_move != curr_move)
            //    board_print(NULL);

            make_move(curr_move);
		}
        printf("\nCount: %d Correct: %d %3.4f\n", total_move_count, total_correct_count, (double)((double)total_correct_count * 100.0 / (double)total_move_count));
        printf("Moves: P=%6d B=%6d N=%6d R=%6d Q=%6d K=%6d\n", piece_moves[PAWN], piece_moves[BISHOP], piece_moves[KNIGHT], piece_moves[ROOK], piece_moves[QUEEN], piece_moves[KING]);
        printf("Wrong: P=%6d B=%6d N=%6d R=%6d Q=%6d K=%6d\n", piece_count[PAWN], piece_count[BISHOP], piece_count[KNIGHT], piece_count[ROOK], piece_count[QUEEN], piece_count[KING]);
        printf("Pct..: P=%6d B=%6d N=%6d R=%6d Q=%6d K=%6d\n", piece_count[PAWN] * 100 / piece_moves[PAWN], 
                                                               piece_count[BISHOP] * 100 / piece_moves[BISHOP], 
                                                               piece_count[KNIGHT] * 100 / piece_moves[KNIGHT], 
                                                               piece_count[ROOK] * 100 / piece_moves[ROOK], 
                                                               piece_count[QUEEN] * 100 / piece_moves[QUEEN], 
                                                               piece_count[KING] * 100 / piece_moves[KING]);
	}

	fclose(pgn);
}

MOVE pgn_get_curr_move(char *pgn_move)
{
    MOVES   ml[1];
    MOVE    move;
    char    desc[40];

    moves_init(ml, is_incheck(board.side_on_move), MOVE_NONE, 0);
    while((move = moves_next(ml)) != MOVE_NONE)
    {
        assert(is_valid(move));
        assert(board_state_is_ok());
        make_move(move);
        if (!is_illegal(move))
        {
            util_pgn_desc(move, desc, 1, 0);
            //printf("DESC=%s\n", desc);

            if (strcmp(pgn_move, desc) != 0)
            {
                util_pgn_desc(move, desc, 0, 1);
            }
            if (strcmp(pgn_move, desc) != 0)
            {
                util_pgn_desc(move, desc, 0, 0);
            }

            if (strcmp(pgn_move, desc) == 0)
            {
                undo_move();
                return move;
            }
        }
        undo_move();
    }
    return MOVE_NONE;
}

int pgn_is_number_or_space(char c1)
{
	if ((c1 >= '0' && c1 <= '9') || c1 == '.' || c1 == ' ')
		return 1;
	return 0;
}

int pgn_is_end_game(char *pgn_moves)
{
	while(*pgn_moves == ' ')
		pgn_moves++;
	if (!*pgn_moves)
		return 1;
	if (!strncmp(pgn_moves, "1-0", 3) || !strncmp(pgn_moves, "0-1", 3) || !strncmp(pgn_moves, "1/2-1/2", 7))
		return 1;
	return 0;
}

#define ADD_SPACE(s, i)  (i > 0 && s[i - 1] != ' ' ? 1 : 0)

// 0-1
// 1-0
// 1/2-1/2
// *
int is_end_game(char *g, int p)
{
    if (p > 0 && g[p-1] == '*')
        return 1;
    if (p > 2 && g[p-1] == '0' && g[p-2] == '-' && g[p-3] == '1')
        return 1;
    if (p > 2 && g[p-1] == '1' && g[p-2] == '-' && g[p-3] == '0')
        return 1;
    if (p > 6 && g[p-1] == '2' && g[p-2] == '/' && g[p-3] == '1' && g[p-4] == '-' && g[p-5] == '2' && g[p-6] == '/' && g[p-7] == '1')
        return 1;
    return 0;
}

int read_game(FILE *pgn, char *game)
{
    int     byte;
    int     pos = 0;
    int     is_comm = 0;
    int     is_tag = 0;

    game[0] = 0;

    while((byte = fgetc(pgn)) != EOF)
    {
        if (is_comm)
        {
            if (byte == '}')
            {
                is_comm = 0;
            }
            continue;
        }
        if (is_tag)
        {
            if (byte == ']')
            {
                is_tag = 0;
            }
            continue;
        }
        if (byte == '{')
        {
            is_comm = 1;
            continue;
        }
        if (byte == '[')
        {
            is_tag = 1;
            continue;
        }
        if (byte == '\n')
        {
            if (ADD_SPACE(game, pos))
                game[pos++] = ' ';
            continue;
        }
        if (byte == ' ')
        {
            if (ADD_SPACE(game, pos))
                game[pos++] = ' ';
            continue;
        }
        game[pos++] = (char)byte;
        if (is_end_game(game, pos))
            break;
    }

    game[pos] = 0;
    if (!strcmp(game, " "))
        game[0] = 0;
    return (int)strlen(game);
}

static const char *FILE_LETTER = "abcdefgh";
static const char *RANK_NUMBER = "87654321";

#define file_letter(sq)    FILE_LETTER[square_file((sq))]
#define rank_number(sq)    RANK_NUMBER[square_rank((sq))]

void util_pgn_desc(MOVE move, char *string, int inc_file, int inc_rank)
{
	char promo_piece;
	char moving_piece[2];

	if (MVPC(move) == PAWN)
    {
        if (IS_QUIET(move))
        {
		    sprintf(string, "%c%c", file_letter(TOSQ(move)), rank_number(TOSQ(move)));
            return;
        }
		moving_piece[0] = 0;
    }
	else
    {
		moving_piece[0] = piece_letter[MVPC(move)];
		moving_piece[1] = 0;
	}
	if (TYPE(move) == MT_PROMO)
    {
		promo_piece = promo_letter[PROM(move)];
		sprintf(string, "%s%c%c=%c", moving_piece,
		                             file_letter(TOSQ(move)), rank_number(TOSQ(move)),
								     toupper(promo_piece));
        return;
	}
	if (TYPE(move) == MT_CPPRM)
    {
		promo_piece = promo_letter[PROM(move)];
		sprintf(string, "%cx%c%c=%c", file_letter(FRSQ(move)),
		                              file_letter(TOSQ(move)), rank_number(TOSQ(move)),
								      toupper(promo_piece));
        return;
    }
	if (TYPE(move) == MT_EPCAP)
    {
		sprintf(string, "%cx%c%c", file_letter(FRSQ(move)), file_letter(TOSQ(move)), rank_number(TOSQ(move)));
        return;
    }
    if (TYPE(move) == MT_CSWKS || TYPE(move) == MT_CSBKS)
    {
        sprintf(string, "O-O");
        return;
    }
    if (TYPE(move) == MT_CSWQS || TYPE(move) == MT_CSBQS)
    {
        sprintf(string, "O-O-O");
        return;
    }
    if (inc_file)
        sprintf(string, "%s%c%s%c%c", moving_piece, file_letter(FRSQ(move)), (TYPE(move) == MT_CAPPC ? "x" : ""), file_letter(TOSQ(move)), rank_number(TOSQ(move)));
    else
    if (inc_rank)
        sprintf(string, "%s%c%s%c%c", moving_piece, rank_number(FRSQ(move)), (TYPE(move) == MT_CAPPC ? "x" : ""), file_letter(TOSQ(move)), rank_number(TOSQ(move)));
    else
    {
        if (MVPC(move) == PAWN && TYPE(move) == MT_CAPPC)
            sprintf(string, "%c%s%c%c", file_letter(FRSQ(move)), (TYPE(move) == MT_CAPPC ? "x" : ""), file_letter(TOSQ(move)), rank_number(TOSQ(move)));
        else
            sprintf(string, "%s%s%c%c", moving_piece, (TYPE(move) == MT_CAPPC ? "x" : ""), file_letter(TOSQ(move)), rank_number(TOSQ(move)));
    }
}

//END