/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//  perft test: count the moves from a position to test move generation.
//-------------------------------------------------------------------------------------------------

U64 perft_nodes(int depth);

void perft(int depth)
{
	UINT		start;
	UINT		finish;
	UINT        duration;
	double		nodes_per_second;
	double		seconds;
	U64         nodes = 0;
	int         d;

	printf("Depth     Nodes Secs Nodes/Sec\n");
    board.ply = 0;

	for (d = 1; d <= depth; d++) 
	{
		start = util_get_time();
		nodes = perft_nodes(d);
		finish = util_get_time();

		duration = (finish - start);
		if (duration == 0)
			duration = 1;
		nodes_per_second = ((double)nodes / (double)duration) * 1000.0;
		seconds = (double)duration / (double)1000;

		printf("%5d ", d);
		printf("%9lld ", nodes);
		printf("%4.1f ", seconds);
		printf("%9.0f ", nodes_per_second);
		printf("\n");
	}
	printf("perft completed.\n");
}

U64 perft_nodes(int depth) {
	U64     nodes = 0;
    int     move;
    MOVES   ml;

	if (depth == 0)
		return 1;

	moves_init(&ml, is_incheck(board.side_on_move), 0, 0);
	
	while((move = moves_next_perft(&ml)))
	{
		make_move(move);

		if (!is_illegal(move))
    		nodes += perft_nodes(depth - 1);

		undo_move();
	}

	return nodes;
}

//END