#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <stdlib.h>

int pgn_read_game(FILE *pgn, char *pgn_moves);
int pgn_is_loss(char *game, char *player);
void pgn_high_score(char *game, char *player, double *score, int *move);

#define MAXMOV 102400
#define PLAYER "tucano105s"

/*

*/

void test_pgn(void)
{
	FILE    *pgn = fopen("a.pgn", "r");
    FILE    *out = fopen("a.csv", "w");

	char    game[MAXMOV];
    int     move;
    double  score;
    int     count = 0;

    //char    xx[100];

	while(pgn_read_game(pgn, game))
	{
        count++;

        if (!pgn_is_loss(game, PLAYER))
            continue;

        printf("%d\n", count);

        pgn_high_score(game, PLAYER, &score, &move);

        fprintf(out, "%3.2f, %d, \"%s\"\n", score, move, game);
	}

	fclose(pgn);
    fclose(out);
}

void pgn_tag(char *game, char *tag, char *val)
{
    int     i;
    int     j;

    val[0] = 0;

    for (i = 0; game[i]; i++)
    {
        if (game[i] == '[' && !strncmp(tag, &game[i+1], strlen(tag)))
        {
            i += (int)strlen(tag) + 1;
            for (j = 0; j < 99 && game[i] && game[i] != ']'; i++)
            {
                if (game[i] == '"' || game[i] == ' ')
                    continue;
                val[j++] = game[i];
            }
            val[j] = 0;
            return;
        }
    }
}

int pgn_is_loss(char *game, char *player)
{
    char white[100];
    char black[100];
    char result[100];

    pgn_tag(game, "White", white);
    pgn_tag(game, "Black", black);
    pgn_tag(game, "Result", result);

    if (!strcmp(white, player) && !strcmp(result, "0-1"))
        return 1;
    if (!strcmp(black, player) && !strcmp(result, "1-0"))
        return 1;
    return 0;
}

int pgn_get_move(char *game, int num, char *move)
{
    char    s1[100];
    char    s2[100];
    char    *p;
    int     i;

    sprintf(s1, "%d. ", num);
    p = strstr(game, s1);
    sprintf(s2, "%d. ", num + 1);
    move[0] = 0;

    if (p)
    {
        for (i = 0; strncmp(p, s2, strlen(s2)) && *p; i++, p++)
        {
            move[i] = *p;
        }
        move[i] = 0;
        return 1;
    }

    return 0;
}

double pgn_move_score(char *move)
{
    char temp[100];
    char *s;
    char *e;
    int i;

    s = strchr(move, '{');
    if (s++ == NULL)
        return 0.0;
    e = strchr(move, '/');
    if (e == NULL)
        return 0.0;
    for (i = 0; i < 99 && s < e; s++, i++)
        temp[i] = *s;
    temp[i] = 0;
    return atof(temp);
}

void pgn_get_moves(char *game, char moves[1000][100])
{
    int     n;

    for(n = 1; n < 1000 && pgn_get_move(game, n, moves[n - 1]); n++);
}

void pgn_high_white(char moves[1000][100], double *score, int *move)
{
    int     i;
    double  temp;

    *score = (double)INT_MIN;
    *move = 0;

    for (i = 0; moves[i][0]; i++)
    {
        temp = pgn_move_score(moves[i]);
        if (temp > *score)
        {
            *score = temp;
            *move = i + 1;
        }
    }
}

void pgn_high_black(char moves[1000][100], double *score, int *move)
{
    int     i;
    double  temp;
    char    *bm;

    *score = (double)INT_MAX;
    *move = 0;

    for (i = 0; moves[i][0]; i++)
    {
        bm = strchr(moves[i], '}') + 1;
        if (bm == NULL)
            continue;
        temp = pgn_move_score(bm);
        if (temp < *score)
        {
            *score = temp;
            *move = i + 1;
        }
    }

    *score = -*score;
}

void pgn_high_score(char *game, char *player, double *score, int *move)
{
    char    white[100];
    char    moves[1000][100];

    pgn_get_moves(game, moves);

    pgn_tag(game, "White", white);

    if (!strcmp(player, white))
        pgn_high_white(moves, score, move);
    else
        pgn_high_black(moves, score, move);

    //printf("HIGH %d %3.2f\n", *move, *score);
}

#define ADD_SPACE(s, i)  (i > 0 && s[i - 1] != ' ' ? 1 : 0)

int pgn_is_end_of_game(char *g, int p)
{
    if (p > 0 && g[p-1] == '*')
        return 1;
    if (p > 2 && g[p-1] == '0' && g[p-2] == '-' && g[p-3] == '1')
        return 1;
    if (p > 2 && g[p-1] == '1' && g[p-2] == '-' && g[p-3] == '0')
        return 1;
    if (p > 6 && g[p-1] == '2' && g[p-2] == '/' && g[p-3] == '1' && g[p-4] == '-' && g[p-5] == '2' && g[p-6] == '/' && g[p-7] == '1')
        return 1;
    return 0;
}

int pgn_read_game(FILE *pgn, char *game)
{
    int     byte;
    int     pos = 0;
    int     tag = 0;

    game[0] = 0;

    while((byte = fgetc(pgn)) != EOF)
    {
        if (byte == '[')
            tag = 1;
        if (byte == '\n')
        {
            if (ADD_SPACE(game, pos))
                game[pos++] = ' ';
            continue;
        }
        if (byte == ' ')
        {
            if (ADD_SPACE(game, pos))
                game[pos++] = ' ';
            continue;
        }
        game[pos++] = (char)byte;

        if (!tag && pgn_is_end_of_game(game, pos))
            break;
        if (byte == ']')
            tag = 0;
    }

    game[pos] = 0;
    if (!strcmp(game, " "))
        game[0] = 0;
    return (int)strlen(game);
}

//END