/*-------------------------------------------------------------------------------
  tucano is a XBoard chess playing engine developed by Alcides Schulz.
  Copyright (C) 2011-present - Alcides Schulz

  tucano is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  tucano is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You can find the GNU General Public License at http://www.gnu.org/licenses/
-------------------------------------------------------------------------------*/

#include "globals.h"

//-------------------------------------------------------------------------------------------------
//	Zobrist keys functions.
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
//	Initialization.
//-------------------------------------------------------------------------------------------------
void zk_init(void) {
	int p, c, i;

	for (c = 0; c < 2; c++)
    {
		zkCastleKS[c][0] = zk_rand();
		zkCastleKS[c][1] = zk_rand();
		zkCastleQS[c][0] = zk_rand();
		zkCastleQS[c][1] = zk_rand();
		for (p = 0; p < 7; p++)
        {
			for (i = 0; i < 64; i++)
            {
				zkBase[c][p][i] = zk_rand();
			}
		}
	}

	zkColor = zk_rand();
	for (i = 0; i < 64; i++)
		zkEnPassant[i] = zk_rand(); // will use only 16 of those
}

//-------------------------------------------------------------------------------------------------
//	Generate a random 64 bit number.
//-------------------------------------------------------------------------------------------------
U64 zk_rand(void)
{
	U64 temp = rand();
	temp ^= ((U64)rand() << 15);
	temp ^= ((U64)rand() << 30);
	temp ^= ((U64)rand() << 45);
	temp ^= ((U64)rand() << 60);
	return temp;
}

//-------------------------------------------------------------------------------------------------
//	Calculate current position key.
//-------------------------------------------------------------------------------------------------
U64 zk_board_key(void)
{
	int i;

	U64 key = 0;

	for (i = 0; i < 64; i++)
    {
        assert(board.state[BLACK].square[i] >= EMPTY && board.state[BLACK].square[i] < NUM_PIECES);
        assert(board.state[WHITE].square[i] >= EMPTY && board.state[WHITE].square[i] < NUM_PIECES);

		if (board.state[BLACK].square[i])
			key ^= zkBase[BLACK][board.state[BLACK].square[i]][i];
		if (board.state[WHITE].square[i])
			key ^= zkBase[WHITE][board.state[WHITE].square[i]][i];
	}
	if (board.side_on_move == BLACK)
		key ^= zkColor;
	if (board.ep_square.u64)
		key ^= zkEnPassant[bb_first(board.ep_square)];
	key ^= zkCastleKS[board.side_on_move][STM_STATE.can_castle_ks];
	key ^= zkCastleQS[board.side_on_move][STM_STATE.can_castle_qs];

	return key;
}

//-------------------------------------------------------------------------------------------------
//	Calculate current position pawn key.
//-------------------------------------------------------------------------------------------------
U64 zk_pawn_key(void)
{
    U64     pkey = 0;
    BBIX    piece;
    int     pcsq;

    piece.u64 = board.state[WHITE].piece[PAWN].u64;
    while(piece.u64)
    {
        pcsq = bb_first(piece);
        pkey ^= zkBase[WHITE][PAWN][pcsq];
        bb_clear(piece.u64, pcsq);
    }
    piece.u64 = board.state[BLACK].piece[PAWN].u64;
    while(piece.u64)
    {
        pcsq = bb_first(piece);
        pkey ^= zkBase[BLACK][PAWN][pcsq];
        bb_clear(piece.u64, pcsq);
    }
    if (board.side_on_move == BLACK)
		pkey ^= zkColor;
    return pkey;
}

//END